Look for XXX XXX XXX
// $Id: $
/* Copyright (c) 2001 The Regents of the University of California through
   E.O. Lawrence Berkeley National Laboratory, subject to approval by the
   U.S. Department of Energy. See files COPYRIGHT.txt and
   cctbx/LICENSE.txt for further details.

   Revision history:
     Created: 2001-Sep-12 (based on sginfo2/sgasym.c) (R.W. Grosse-Kunstleve)
 */

#include <cctbx/sgtbx/groups.h>

#ifdef JUNK

#define CoV(nGrid, Ix, V) { (V)[0] = (Ix) / (nGrid) / (nGrid);\
                            (V)[1] = (Ix) / (nGrid) % (nGrid);\
                            (V)[2] = (Ix) % (nGrid);\
                          }

static int BrickVolume(int Brick[2][3])
{
  int  Vol, i;
  Vol = 1; do (i, 0, 3) Vol *= (Brick[1][i] - Brick[0][i]);
  return Vol;
}

static int CmpEqVolBricks(int nGrid, const int PseudoG[9],
                          int a[2][3], int b[2][3])
{
  int  na, nb, i;

  na = nb = 1;
  do (i, 0, 2) na *= (a[1][i] - a[0][i]);
  do (i, 0, 2) nb *= (b[1][i] - b[0][i]);
  if (na < nb) return -1;
  if (na > nb) return  1;

  na = nb = 0;
  do (i, 0, 3) if (a[1][i] > nGrid) na++;
  do (i, 0, 3) if (b[1][i] > nGrid) nb++;
  if (na < nb) return -1;
  if (na > nb) return  1;

  na = iScalProd(a[0], a[0], PseudoG);
  nb = iScalProd(b[0], b[0], PseudoG);
  if (na < nb) return -1;
  if (na > nb) return  1;

  na = nb = 0;
  do (i, 0, 3) if (a[0][i] == 0) na++;
  do (i, 0, 3) if (b[0][i] == 0) nb++;
  if (na > nb) return -1;
  if (na < nb) return  1;

  na = nb = 0;
  do (i, 0, 3) if (a[0][i] >= 0) na++;
  do (i, 0, 3) if (b[0][i] >= 0) nb++;
  if (na > nb) return -1;
  if (na < nb) return  1;

  do (i, 0, 3) {
    if (abs(a[0][i]) < abs(b[0][i])) return -1;
    if (abs(a[0][i]) > abs(b[0][i])) return  1;
  }

  do (i, 0, 3) {
    if (a[0][i] < b[0][i]) return -1;
    if (a[0][i] > b[0][i]) return  1;
  }

  return 0;
}

static int iFloor(int n, int d) /* d must be greater than 0 */
{
  int  i;

  i = n / d;
  if (n < 0 && n % d)
    i--;

  return i;
}

static int PointIsInBrick(int nGrid, int Brick[2][3], const int Point[3])
{
  int  i, n, p;

  do (i, 0, 3) {
    n = iFloor(Point[i] - Brick[0][i], nGrid);
    p = Point[i] - n * nGrid;
    if (p <  Brick[0][i])         return IE(-1);
    if (p >= Brick[0][i] + nGrid) return IE(-1);
    if (p >  Brick[1][i]) return 0;
  }

  return 1;
}

typedef struct
  {
    int  Status;
    int  nFace;
    int  nEdge;
    int  nVert;
  }
  T_GrowFld;

static void InitGrowFld(int nGrid, T_GrowFld GrowFld[])
{
  int  i;

  do (i, 0, ToThe3(nGrid)) {
    GrowFld[i].Status = 0;
    GrowFld[i].nFace = 0;
    GrowFld[i].nEdge = 0;
    GrowFld[i].nVert = 0;
  }
}

static int GF_MarkAsym(int nGrid, T_GrowFld GrowFld[], int PivotIx)
{
  int        P[3], Offs[3], PO[3], n, i;
  T_GrowFld  *GF;


      GF = &GrowFld[PivotIx];
  if (GF->Status != 0) return IE(-1);
      GF->Status = 1;

  CoV(nGrid, PivotIx, P);

  for (Offs[0] = -1; Offs[0] <= 1; Offs[0]++)
  for (Offs[1] = -1; Offs[1] <= 1; Offs[1]++)
  for (Offs[2] = -1; Offs[2] <= 1; Offs[2]++)
  {
    do (i, 0, 3) PO[i] = P[i] + Offs[i];
        GF = &GrowFld[IxV(nGrid, PO)];
    if (GF->Status == 0) {
      n = 0; do (i, 0, 3) if (Offs[i]) n++;
      switch (n) {
        case 1: GF->nFace++; break;
        case 2: GF->nEdge++; break;
        case 3: GF->nVert++; break;
        default: break;
      }
    }
  }

  return 0;
}


static int GF_MarkSym(const int OrbCyc[], T_GrowFld GrowFld[], int PivotIx)
{
  int  PIx, NextIx;


  PIx = PivotIx;

  for (NextIx = OrbCyc[PIx]; NextIx != PIx; NextIx = OrbCyc[NextIx]) {
    if (GrowFld[NextIx].Status != 0) return IE(-1);
        GrowFld[NextIx].Status = -1;
  }

  return 0;
}


static int CountSymEqInAU(int nGrid, const int OrbCyc[], int Brick[2][3],
                          int PIx)
{
  int  nInAU, NextIx, N[3], InAU;


  nInAU = 0;

  for (NextIx = OrbCyc[PIx]; NextIx != PIx; NextIx = OrbCyc[NextIx]) {
    CoV(nGrid, NextIx, N);
        InAU = PointIsInBrick(nGrid, Brick, N);
    if (InAU < 0) return IE(-1);
    if (InAU != 0)
       nInAU++;
  }

  return nInAU;
}


static int GF_SetMandatory(int nGrid, const int OrbCyc[], int Brick[2][3],
                           T_GrowFld GrowFld[])
{
  int        P[3], PIx, nInAU;
  T_GrowFld  *GF;


  for (P[0] = Brick[0][0]; P[0] <= Brick[1][0]; P[0]++)
  for (P[1] = Brick[0][1]; P[1] <= Brick[1][1]; P[1]++)
  for (P[2] = Brick[0][2]; P[2] <= Brick[1][2]; P[2]++) {
    PIx = IxV(nGrid, P);
    GF = &GrowFld[PIx];
    if (GF->Status == 0) {
          nInAU = CountSymEqInAU(nGrid, OrbCyc, Brick, PIx);
      if (nInAU < 0) return IE(-1);
      if (nInAU == 0) {
        if (GF_MarkAsym(nGrid, GrowFld, NULL, PIx) != 0)
          return IE(-1);
        if (GF_MarkSym(OrbCyc, GrowFld, PIx) != 0)
          return IE(-1);
      }
    }
  }

  return 0;
}


static int GF_FixBrokenRods(int nGrid, const int OrbCyc[], int Brick[2][3],
                            T_GrowFld GrowFld[])
{
  int  P[3], PIx, RodMin, RodMax, RIx;


for (P[0] = Brick[0][0]; P[0] <= Brick[1][0]; P[0]++)
for (P[1] = Brick[0][1]; P[1] <= Brick[1][1]; P[1]++)
for (P[2] = Brick[0][2]; P[2] <= Brick[1][2]; P[2]++) {
PIx = IxV(nGrid, P);
if (GrowFld[PIx].Status < 0) return IE(-1);
}

  for (P[0] = Brick[0][0]; P[0] <= Brick[1][0]; P[0]++)
  for (P[1] = Brick[0][1]; P[1] <= Brick[1][1]; P[1]++)
  {
    for (P[2] = Brick[0][2]; P[2] <= Brick[1][2]; P[2]++) {
      PIx = IxV(nGrid, P);
      if (GrowFld[PIx].Status > 0) break;
    }

    if (P[2] > Brick[1][2])
      continue;

    RodMin = RodMax = P[2];

    for (P[2]++            ; P[2] <= Brick[1][2]; P[2]++) {
      PIx = IxV(nGrid, P);
      if (GrowFld[PIx].Status > 0)
        RodMax = P[2];
    }

    for (P[2] = RodMin; P[2] <= RodMax; P[2]++) {
      PIx = IxV(nGrid, P);
      if (GrowFld[PIx].Status < 0) return 0;
      if (GrowFld[PIx].Status == 0) {
        if (GF_MarkAsym(nGrid, GrowFld, NULL, PIx) != 0)
          return IE(-1);
        if (GF_MarkSym(OrbCyc, GrowFld, PIx) != 0)
          return IE(-1);
      }
    }
  }

  return 1;
}



static int IsOnSrchGr(int nGrid, int i)
{
  if (i % (nGrid / 12) && i % (nGrid / 8)) return 0;
  return 1;
}


static int VectIsOnSrchGr(int nGrid, const int P[3])
{
  int  i;

  do (i, 0, 3)
    if (! IsOnSrchGr(nGrid, P[i]))
      return 0;

  return 1;
}


static int SetLSrchGr(int nGrid, int LSrchGr[])
{
  int   nLSrchGr, i;
#define mLSrchGr (33)


  nLSrchGr = 0;

  do (i, 0, 2 * nGrid + 1) {
    if (IsOnSrchGr(nGrid, i)) {
      if (nLSrchGr >= mLSrchGr) return IE(-1);
      LSrchGr[nLSrchGr] = i;
              nLSrchGr++;
    }
  }

  if (nLSrchGr != mLSrchGr) return IE(-1);

  return nLSrchGr;
}


static int SweepSlices(int nGrid, const int OrbCyc[], int UFlags[],
                       int *SweptPlanes[3],
                       int Brick[2][3], int BrickIsSet[3], int SweptHigh[3],
                       int Orientation)
{
  int  i, AUBrick[2][3], SwBrick[2][3], P[3], PIx, NextIx, N[3], InAU;


  MemCpy(AUBrick, Brick, 2);

  do (i, 0, 3) {
    if (BrickIsSet[i] == 0) {
      AUBrick[0][i] = 0;
      AUBrick[1][i] = nGrid - 1;
    }
  }

  MemCpy(SwBrick, AUBrick, 2);

  SwBrick[0][Orientation] = AUBrick[1][Orientation] + 1;
  SwBrick[1][Orientation] = SweptHigh[Orientation] - 1;

  if (SwBrick[0][Orientation] > SwBrick[1][Orientation])
    return 0;

  if      (Orientation == 0)
  {
    for (P[0] = SwBrick[1][0]; P[0] >= SwBrick[0][0]; P[0]--) {
      if (SweptPlanes[0][ModPos(P[0], nGrid)]) continue;
      for (P[1] = SwBrick[0][1]; P[1] <= SwBrick[1][1]; P[1]++) {
        if (SweptPlanes[1][ModPos(P[1], nGrid)]) continue;
        for (P[2] = SwBrick[0][2]; P[2] <= SwBrick[1][2]; P[2]++) {
          if (SweptPlanes[2][ModPos(P[2], nGrid)]) continue;
                     PIx = IxV(nGrid, P);
          if (UFlags[PIx] != 0) {
            for (NextIx = OrbCyc[PIx]; NextIx != PIx;
                 NextIx = OrbCyc[NextIx]) {
              CoV(nGrid, NextIx, N);
                  InAU = PointIsInBrick(nGrid, AUBrick, N);
              if (InAU < 0) return IE(-1);
              if (InAU != 0) {
                UFlags[NextIx] = UFlags[PIx];
                UFlags[PIx] = 0;
                do (i, 0, 3) SweptPlanes[i][N[i]] = 0;
                break;
              }
            }
            if (UFlags[PIx] != 0)
              return 1;
          }
        }
      }
      SweptPlanes[0][ModPos(P[0], nGrid)] = 1;
      SweptHigh[0] = P[0];
    }
  }
  else if (Orientation == 1)
  {
    for (P[1] = SwBrick[1][1]; P[1] >= SwBrick[0][1]; P[1]--) {
      if (SweptPlanes[1][ModPos(P[1], nGrid)]) continue;
      for (P[0] = SwBrick[0][0]; P[0] <= SwBrick[1][0]; P[0]++) {
        if (SweptPlanes[0][ModPos(P[0], nGrid)]) continue;
        for (P[2] = SwBrick[0][2]; P[2] <= SwBrick[1][2]; P[2]++) {
          if (SweptPlanes[2][ModPos(P[2], nGrid)]) continue;
                     PIx = IxV(nGrid, P);
          if (UFlags[PIx] != 0) {
            for (NextIx = OrbCyc[PIx]; NextIx != PIx;
                 NextIx = OrbCyc[NextIx]) {
              CoV(nGrid, NextIx, N);
                  InAU = PointIsInBrick(nGrid, AUBrick, N);
              if (InAU < 0) return IE(-1);
              if (InAU != 0) {
                UFlags[NextIx] = UFlags[PIx];
                UFlags[PIx] = 0;
                do (i, 0, 3) SweptPlanes[i][N[i]] = 0;
                break;
              }
            }
            if (UFlags[PIx] != 0)
              return 1;
          }
        }
      }
      SweptPlanes[1][ModPos(P[1], nGrid)] = 1;
      SweptHigh[1] = P[1];
    }
  }
  else if (Orientation == 2)
  {
    for (P[2] = SwBrick[1][2]; P[2] >= SwBrick[0][2]; P[2]--) {
      if (SweptPlanes[2][ModPos(P[2], nGrid)]) continue;
      for (P[0] = SwBrick[0][0]; P[0] <= SwBrick[1][0]; P[0]++) {
        if (SweptPlanes[0][ModPos(P[0], nGrid)]) continue;
        for (P[1] = SwBrick[0][1]; P[1] <= SwBrick[1][1]; P[1]++) {
          if (SweptPlanes[1][ModPos(P[1], nGrid)]) continue;
                     PIx = IxV(nGrid, P);
          if (UFlags[PIx] != 0) {
            for (NextIx = OrbCyc[PIx]; NextIx != PIx;
                 NextIx = OrbCyc[NextIx]) {
              CoV(nGrid, NextIx, N);
                  InAU = PointIsInBrick(nGrid, AUBrick, N);
              if (InAU < 0) return IE(-1);
              if (InAU != 0) {
                UFlags[NextIx] = UFlags[PIx];
                UFlags[PIx] = 0;
                do (i, 0, 3) SweptPlanes[i][N[i]] = 0;
                break;
              }
            }
            if (UFlags[PIx] != 0)
              return 1;
          }
        }
      }
      SweptPlanes[2][ModPos(P[2], nGrid)] = 1;
      SweptHigh[2] = P[2];
    }
  }
  else
    return IE(-1);

  return 0;
}


static int SetOffBrick(int nGrid, int Brick[2][3],
                       int nGrid2, const int OrbCyc2[], int UFlags2[],
                       int *SweptPlanes2[3],
                       int OffBrick[2][3])
{
  int  fG, BestVol, Vol, iBV, n, i;
  int  Off[2][3], Brick2[2][3], BrickIsSet[3], Swept[3];


  fG = nGrid2 / nGrid;

  BestVol = -1;

  do (Off[0][0], 0, 2)
  do (Off[0][1], 0, 2)
  do (Off[0][2], 0, 2)
  do (Off[1][0], 0, 2)
  do (Off[1][1], 0, 2)
  do (Off[1][2], 0, 2)
  {
    Vol = 1;
    do (iBV, 0, 3) {
      Vol *= (  fG * (Brick[1][iBV] - Brick[0][iBV] + 1)
              - Off[0][iBV]
              - Off[1][iBV]);
    }

    if (BestVol > Vol || BestVol < 0) {
      IntSetZero(BrickIsSet, 3);
      do (iBV, 0, 3) {
        do (i, 0, 2) {
          Brick2[i][iBV] = Brick[i][iBV] * fG;
          if (Off[i][iBV]) Brick2[i][iBV] += (i ? -1 : 1);
        }
        BrickIsSet[iBV] = 1;
        Swept[iBV] = Brick2[0][iBV] + nGrid2;
            n = SweepSlices(nGrid2, OrbCyc2, UFlags2, SweptPlanes2,
                            Brick2, BrickIsSet, Swept, iBV);
        if (n < 0) return IE(-1);
        if (n > 0) break;
      }
      if (iBV == 3) {
        MemCpy(OffBrick, Off, 2);
        BestVol = Vol;
      }
    }
  }

  if (BestVol < 0) return IE(-1);

  return 0;
}


static int AOShiftBrick(int nGrid,
                        const int OrbCycAOS[], const int PseudoG[3 * 3],
                        int Brick[2][3])
{
  int  i, D[3], FirstIx, NextIx, N[3], USh[3], ShBrick[2][3];


  do (i, 0, 3) D[i] = Brick[1][i] - Brick[0][i];

  FirstIx = NextIx = IxV(nGrid, Brick[0]);

  for (;;) {
    CoV(nGrid, NextIx, N);
    do (USh[0], 0, 2)
    do (USh[1], 0, 2)
    do (USh[2], 0, 2) {
      do (i, 0, 3) {
        ShBrick[0][i] = N[i] - USh[i] * nGrid;
        ShBrick[1][i] = ShBrick[0][i] + D[i];
      }
      if (CmpEqVolBricks(nGrid, PseudoG, Brick, ShBrick) > 0) {
        MemCpy(Brick, ShBrick, 2);
        do (i, 0, 3) if (Brick[0][i] != 0) goto NextAOS;
      }
    }
    NextAOS:
        NextIx = OrbCycAOS[NextIx];
    if (NextIx == FirstIx) break;
  }

  return 0;
}


static int FillAdjBrick(int nGrid, const int OrbCyc[], int Brick[2][3],
                        T_GrowFld GrowFld[])
{
  InitGrowFld(nGrid, GrowFld);

  if (GF_SetMandatory(nGrid, OrbCyc, Brick, GrowFld) != 0)
    return IE(-1);

  return GF_FixBrokenRods(nGrid, OrbCyc, Brick, GrowFld);
}


static int AdjBrickMargins(int nGrid, int Brick[2][3],
                           int nGrid2, const int OrbCyc2[], int UFlags2[],
                           int *SweptPlanes2[3],
                           int nLSrchGr, int LSrchGr[],
                           const int OrbCycAOS[], const int PseudoG[3 * 3],
                           int nBest, int BestBrick[2][3],
                           const int OrbCyc[], T_GrowFld GrowFld[])
{
  int  iBV, fG, gcd0, gcd1, HaveBetter, AdjVol, i, n;
  int  AdjIx[3], AdjDir[3], Off[3];
  int  AdjBrick[2][3], Brick2[2][3], BrickIsSet[3], SweptHigh[3];
  int  SE[2][3];


  fG = nGrid2 / nGrid;

  do (iBV, 0, 3) {
    do (i, 0, 2) {
      do (SE[i][iBV], 0, nLSrchGr)
        if (LSrchGr[SE[i][iBV]] == Brick[i][iBV]) break;
      if (SE[i][iBV] ==  nLSrchGr) return IE(-1);
    }
    gcd0 = iGCD(LSrchGr[SE[0][iBV] - 1], nGrid);
    gcd1 = iGCD(LSrchGr[SE[1][iBV] + 1], nGrid);
    if (gcd0 > gcd1) { AdjIx[iBV] = 0; AdjDir[iBV] = -1; }
    else             { AdjIx[iBV] = 1; AdjDir[iBV] =  1; }
  }

  HaveBetter = 0;

  do (Off[0], 0, 2)
  do (Off[1], 0, 2)
  do (Off[2], 0, 2)
  {
    MemCpy(AdjBrick, Brick, 2);
    do (iBV, 0, 3) {
      if (Off[iBV]) AdjBrick[AdjIx[iBV]][iBV] =   LSrchGr[SE[AdjIx[iBV]][iBV]
                                                + AdjDir[iBV]];
    }
                  AdjVol = BrickVolume(AdjBrick);
              if (AdjVol < 1) return IE(-1);
    if (nBest >= AdjVol)
    {
      IntSetZero(BrickIsSet, 3);
      do (iBV, 0, 3) {
        do (i, 0, 2) Brick2[i][iBV] = fG * AdjBrick[i][iBV];
        if (Off[iBV]) Brick2[AdjIx[iBV]][iBV] -= AdjDir[iBV];
        BrickIsSet[iBV] = 1;
        SweptHigh[iBV] = Brick2[0][iBV] + nGrid2;
            n = SweepSlices(nGrid2, OrbCyc2, UFlags2, SweptPlanes2,
                            Brick2, BrickIsSet, SweptHigh, iBV);
        if (n < 0) return IE(-1);
        if (n > 0) break;
      }
      if (iBV == 3) {
        if (AOShiftBrick(nGrid, OrbCycAOS, PseudoG, AdjBrick) != 0)
          return IE(-1);
        if (   nBest >  AdjVol
            || CmpEqVolBricks(nGrid, PseudoG, BestBrick, AdjBrick) > 0) {
              n = FillAdjBrick(nGrid, OrbCyc, AdjBrick, GrowFld);
          if (n < 0) return IE(-1);
          if (n > 0) {
            MemCpy(BestBrick, AdjBrick, 2);
            nBest = AdjVol;
            HaveBetter = 1;
          }
        }
      }
    }
  }

  if (HaveBetter == 0)
    return 0;

  return nBest;
}


static int CoreFindBrick(int nGrid, const int UFlagsAOS[],
                         const int OrbCyc[], int UFlags[], int *SweptPlanes[3],
                         int nGrid2, const int OrbCyc2[], int UFlags2[],
                         int *SweptPlanes2[3],
                         const int OrbCycAOS[], const int PseudoG[3 * 3],
                         T_GrowFld GrowFld[],
                         int BestBrick[2][3])
{
  int  i, n;
  int  nBest, HaveBest;
  int  PIx, Brick[2][3], BrickIsSet[3], Swept[3];
  int  S[3], X[3], XOK;
  int  nLSrchGr, LSrchGr_1[mLSrchGr + 1], *LSrchGr;
  int  pLSrchGr;


  LSrchGr = &LSrchGr_1[1];
      nLSrchGr = SetLSrchGr(nGrid, LSrchGr);
  if (nLSrchGr < 0) return IE(-1);
  LSrchGr_1[0] = -LSrchGr[1];

  do (pLSrchGr, 0, nLSrchGr) if (LSrchGr[pLSrchGr] == nGrid) break;

  nBest = ToThe3(nGrid) + 1;
  HaveBest = 0;

  if (UFlagsAOS[0] != 2) return IE(-1);

  do (PIx, 0, ToThe3(nGrid)) {
    if (UFlagsAOS[PIx] == 2) {
      CoV(nGrid, PIx, Brick[0]);
      do (i, 0, 3) {
        do (S[i], 0, pLSrchGr) if (LSrchGr[S[i]] == Brick[0][i]) break;
        if (S[i] ==  pLSrchGr) return IE(-1);
      }
      Swept[0] = Brick[0][0] + nGrid;
      for (X[0] = pLSrchGr - (Brick[0][0] ? 1 : 0); X[0] >= 0; X[0]--) {
        Brick[1][0] = LSrchGr[S[0] + X[0]];
        BrickIsSet[0] = 1;
        BrickIsSet[1] = 0;
        BrickIsSet[2] = 0;
            n = SweepSlices(nGrid, OrbCyc, UFlags, SweptPlanes,
                            Brick, BrickIsSet, Swept, 0);
        if (n < 0) return IE(-1);
        if (n > 0) break;
        Swept[1] = Brick[0][1] + nGrid;
        for (X[1] = pLSrchGr - (Brick[0][1] ? 1 : 0); X[1] >= 0; X[1]--) {
          Brick[1][1] = LSrchGr[S[1] + X[1]];
          BrickIsSet[1] = 1;
          BrickIsSet[2] = 0;
              n = SweepSlices(nGrid, OrbCyc, UFlags, SweptPlanes,
                              Brick, BrickIsSet, Swept, 1);
          if (n < 0) return IE(-1);
          if (n > 0) break;
          XOK = -1;
          Swept[2] = Brick[0][2] + nGrid;
          for (X[2] = pLSrchGr - (Brick[0][2] ? 1 : 0); X[2] >= 0; X[2]--) {
            Brick[1][2] = LSrchGr[S[2] + X[2]];
            BrickIsSet[2] = 1;
                n = SweepSlices(nGrid, OrbCyc, UFlags, SweptPlanes,
                                Brick, BrickIsSet, Swept, 2);
            if (n < 0) return IE(-1);
            if (n > 0) break;
            XOK = X[2];
          }
          if (XOK < 0) break;
          X[2] = XOK;
          Brick[1][2] = LSrchGr[S[2] + X[2]];
              n = BrickVolume(Brick);
          if (n < 1) return IE(-1);
          if (n <= nBest) {
                n = AdjBrickMargins(nGrid, Brick,
                                    nGrid2, OrbCyc2, UFlags2, SweptPlanes2,
                                    nLSrchGr, LSrchGr,
                                    OrbCycAOS, PseudoG,
                                    nBest, BestBrick,
                                    OrbCyc, GrowFld);
            if (n < 0) return IE(-1);
            if (n > 0) {
              if (n > nBest) return IE(-1);
              nBest = n;
              HaveBest = 1;
            }
          }
        }
      }
    }
  }

  if (HaveBest == 0) return IE(-1);

  return 0;
}


static int ModifyAOSUFlags(int nGrid, int OrbCycAOS[], int UFlagsAOS[])
{
  int  nActive, PIx, P[3], NextIx, N[3];


  nActive = 0;

  PIx = 0;
  do (P[0], 0, nGrid)
  do (P[1], 0, nGrid)
  do (P[2], 0, nGrid) {
    if (UFlagsAOS[PIx] == 1) {
      if (VectIsOnSrchGr(nGrid, P)) {
        UFlagsAOS[PIx] = 2;
        nActive++;
      }
      else {
        UFlagsAOS[PIx] = -1;
        for (NextIx = OrbCycAOS[PIx]; NextIx != PIx;
             NextIx = OrbCycAOS[NextIx]) {
          CoV(nGrid, NextIx, N);
          if (VectIsOnSrchGr(nGrid, N)) {
            UFlagsAOS[PIx] = 0;
            UFlagsAOS[NextIx] = 2;
            nActive++;
            break;
          }
        }
      }
    }
    PIx++;
  }

  return nActive;
}

#endif // JUNK

namespace sgtbx {

  namespace detail_find_brick {

    inline std::size_t ModPos(int i, std::size_t j) {
      return (((i % j) + j) % j);
    }

    template <typename PointType, std::size_t Dim>
    class ModGrid : public std::vector<PointType>
    {
      public:
        ModGrid(const boost::array<std::size_t, Dim>& Sampling)
          : m_size(0)
        {
          m_Sampling.assign(0);
        }
        ModGrid(const boost::array<std::size_t, Dim>& Sampling)
          : m_Sampling(Sampling), m_size(1)
        {
          for (std::size_t is=0; is < Dim; is++) {
            m_size *= Sampling[is];
          }
          resize(m_size);
        }
        std::size_t size() { return m_size(); }
        std::size_t iVector(boost::array<int, Dim>& Index)
        {
          std::size_t iv = 0;
          if (Dim) {
            for (std::size_t is = 0;; is++) {
              iv += ModPos(Index[is]);
              if (is + 1 == Dim) break;
              iv *= Sampling[is];
            }
          }
          return iv;
        }
        const PointType&
        operator()(const boost::array<int, Dim>& Index) const {
          return operator[](iVector(Index));
        }
        PointType&
        operator()(const boost::array<int, Dim>& Index) {
          return operator[](iVector(Index));
        }
      private:
        boost::array<std::size_t, Dim> m_Sampling;
        std::size_t m_size;
    };

    void MarkOneSgOpsOrbitCycle(const SpaceGroup& SgOps,
                                ModGrid<int, 3>& OrbCyc,
                                ModGrid<int, 3>& UFlags,
                                const GridPoint& Pivot, // XXX How?
                                std::size_t PIx)
    {
      std::size_t PrevIx = PIx;
      int Mult = 1;
      for (int i=0;i<SgOps.OrderZ();i++) {
        GridPoint Mate = SgOps(i) * Pivot; // XXX How?
        std::size_t NextIx = Mate.iVector();
        if (UFlags[NextIx] != 0 && NextIx != PIx) {
            UFlags[NextIx] = 0;
          OrbCyc[NextIx] = OrbCyc[PrevIx];
          OrbCyc[PrevIx] = NextIx;
          PrevIx = NextIx;
          Mult++;
        }
      }
      cctbx_assert(SgOps.OrderZ() % Mult == 0);
    }

    void MarkSgOpsOrbitCycles(const SpaceGroup& SgOps,
                              ModGrid<int, 3>& OrbCyc,
                              ModGrid<int, 3>& UFlags)
    {
      boost::array<int, 3> P; // XXX use NestedLoop
      std::size_t PIx = 0;
      for (P[0] = 0; P[0] < OrbCyc.Sampling()[0]; P[0]++)
      for (P[1] = 0; P[1] < OrbCyc.Sampling()[1]; P[1]++)
      for (P[2] = 0; P[2] < OrbCyc.Sampling()[2]; P[2]++) {
        if (UFlags[PIx] != 0) {
          MarkOneSgOpsOrbitCycle(SgOps, OrbCyc, UFlags, P, PIx);
        }
        PIx++;
      }
    }

    void SetAOSOrbitCycles(const SpaceGroup& SgOps,
                           ModGrid<int, 3>& OrbCycAOS,
                           ModGrid<int, 3>& UFlagsAOS)
    {
      //XXX use DerivedGroup?
      SpaceGroup LTrSgOps;
      for (int iLTr = 0; iLTr < SgOps.nLTr(); iLTr++) {
        LTrSgOps.expandLTr(SgOps.LTr(i));
      }
      cctbx_assert(LTrSgOps.nLTr() == SgOps.nLTr());
      for (std::size_t i=0;i<OrbCycAOS.size();i++) OrbCyc[i] = i;
      std::fill(UFlagsAOS.begin(), UFlagsAOS.end(), 1);
      MarkSgOpsOrbitCycles(LTrSgOps, OrbCycAOS, UFlagsAOS);
      StructureSeminvariant ss(SgOps);
      if (ss.size() > 0) {
        //MarkAOSOrbitCycles(ss, OrbCycAOS, UFlagsAOS);
      }
    }

  } // namespace detail_find_brick

  void Brick::GridSearch(const SpaceGroup& SgOps)
  {
    const int nGrid = 24;
    const boost::array<std::size_t, 3> Sampling = {nGrid, nGrid, nGrid};

    ModGrid<int, 3> OrbCycAOS(Sampling);
    ModGrid<int, 3> UFlagsAOS(Sampling);
    SetAOSOrbitCycles(SgOps, OrbCycAOS, UFlagsAOS);
    //ModifyAOSUFlags(OrbCycAOS, UFlagsAOS);

    ModGrid<int, 3> OrbCyc(Sampling);
    ModGrid<int, 3> UFlags(Sampling);
    std::size_t i;
    for (i=0;i<OrbCyc.size();i++) OrbCyc[i] = i;
    std::fill(UFlags.begin(), UFlags.end(), 1);
    MarkSgOpsOrbitCycles(SgOps, OrbCyc, UFlags);

    const int nGrid2 = nGrid * 3;
    const boost::array<std::size_t, 3> Sampling2 = {nGrid2, nGrid2, nGrid2};

    ModGrid<int, 3> OrbCyc2(Sampling2);
    ModGrid<int, 3> UFlags2(Sampling2);
    for (i=0;i<OrbCyc2.size();i++) OrbCyc[i] = i;
    std::fill(UFlags2.begin(), UFlags2.end(), 1);
    MarkSgOpsOrbitCycles(SgOps, OrbCyc2, UFlags2);

#ifdef JUNK
    boost::array<boost::array<int, nGrid>, 3> SweptPlanes;
    for(i=0;i<3;i++) SweptPlanes.assign(0);
    boost::array<boost::array<int, nGrid2>, 3> SweptPlanes;
    for(i=0;i<3;i++) SweptPlanes2.assign(0);
    PseudoG = DummyMetricalMatrix(SgOps);
    CoreGridSearch(UFlagsAOS, OrbCyc, UFlags, SweptPlanes,
                   OrbCyc2, UFlags2, SweptPlanes2,
                   OrbCycAOS, PseudoG, /* GrowFld */, BestBrick);
    SetOffBrick(OrbCyc2, UFlags2, SweptPlanes2);
#endif
  }

} // namespace sgtbx
