#include <cstddef>
#include <vector>

namespace matrix {

  template <typename T, std::size_t NR, std::size_t NC>
  class small {
    public:
      typedef T value_type;
      static const int nr = NR;
      static const int nc = NC;
      T elems[NR * NC];
      const T& operator()(std::size_t ir, std::size_t ic) const {
        return elems[ir * nc + ic];
      }
      T& operator()(std::size_t ir, std::size_t ic) {
        return elems[ir * nc + ic];
      }
  };

  template <typename T>
  class big {
    public:
      typedef T value_type;
      int nr;
      int nc;
      big() : nr(0), nc(0) {}
      big(std::size_t NR, std::size_t NC) : nr(NR), nc(NC), elems(NR * NC) {}
      std::vector<T> elems;
      const T& operator()(std::size_t ir, std::size_t ic) const {
        return elems[ir * nc + ic];
      }
      T& operator()(std::size_t ir, std::size_t ic) {
        return elems[ir * nc + ic];
      }
  };

  template <typename T, std::size_t NR, std::size_t NC,
            typename U>
  big<U> operator*(const small<T, NR, NC>& lhs,
                   const big<U>& rhs)
  {
    big<U> result(lhs.nr, rhs.nc);
    for (std::size_t lr = 0; lr < lhs.nr; lr++) {
      for (std::size_t rc = 0; rc < rhs.nc; rc++) {
        result(lr, rc) = 0;
        for (std::size_t lc = 0; lc < lhs.nc; lc++) {
          result(lr, rc) += lhs(lr, lc) * rhs(lc, rc);
        }
      }
    }
    return result;
  }

  template <typename T, std::size_t NR, std::size_t NC,
            typename U>
  big<U> operator*(const big<U>& rhs,
                   const small<T, NR, NC>& lhs)
  {
  }
}

int main()
{
  matrix::small<int, 3, 3> s;
  matrix::big<int> b(3, 3);
  matrix::big<int> r;
  r = s * b;
  return 0;
}
